package com.fjhx.file.util;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.extra.spring.SpringUtil;
import com.baomidou.dynamic.datasource.toolkit.DynamicDataSourceContextHolder;
import com.fjhx.file.entity.FileInfo;
import com.fjhx.file.entity.ObsFile;
import com.fjhx.file.service.FileInfoService;
import com.ruoyi.common.constant.DatasourceConstant;
import com.ruoyi.common.core.domain.BaseIdPo;

import java.util.List;
import java.util.stream.Collectors;

public class ObsFileUtil {

    private static final FileInfoService fileInfoService = SpringUtil.getBean(FileInfoService.class);
    private static final int defaultFileType = 0;

    /**
     * 保存文件
     *
     * @param obsFileList  文件列表
     * @param businessId   业务id
     * @param businessType 业务文件类型
     */
    public static void saveFile(List<ObsFile> obsFileList, Long businessId, Integer businessType) {
        // 切换到从库
        DynamicDataSourceContextHolder.push(DatasourceConstant.SLAVE_NAME);

        if (obsFileList.size() == 0) {
            return;
        }

        List<FileInfo> fileInfoList = BeanUtil.copyToList(obsFileList, FileInfo.class);
        for (FileInfo fileInfo : fileInfoList) {
            fileInfo.setBusinessId(businessId);
            fileInfo.setBusinessType(businessType);
        }
        fileInfoService.updateBatchById(fileInfoList);

        DynamicDataSourceContextHolder.clear();
    }

    /**
     * 保存文件
     *
     * @param obsFileList 文件列表
     * @param businessId  业务id
     */
    public static void saveFile(List<ObsFile> obsFileList, Long businessId) {
        saveFile(obsFileList, businessId, defaultFileType);
    }

    /**
     * 更新文件
     *
     * @param obsFileList  文件列表
     * @param businessId   业务id
     * @param businessType 业务文件类型
     */
    public static void editFile(List<ObsFile> obsFileList, Long businessId, Integer businessType) {
        DynamicDataSourceContextHolder.push(DatasourceConstant.SLAVE_NAME);

        if (obsFileList.size() == 0) {
            return;
        }

        // 删除不包含在此列表的业务id文件
        List<Long> obsFileId = obsFileList.stream().map(ObsFile::getId).collect(Collectors.toList());
        fileInfoService.remove(q -> q.notIn(BaseIdPo::getId, obsFileId).eq(FileInfo::getBusinessId, businessId));

        // 更新在此列表的文件
        saveFile(obsFileList, businessId, businessType);
        DynamicDataSourceContextHolder.clear();
    }

    /**
     * 更新文件
     *
     * @param obsFileList 文件列表
     * @param businessId  业务id
     */
    public static void editFile(List<ObsFile> obsFileList, Long businessId) {
        editFile(obsFileList, businessId, defaultFileType);
    }

    /**
     * 删除文件
     *
     * @param businessId   业务id
     * @param businessType 业务文件类型
     */
    public static void removeFile(Long businessId, Integer businessType) {
        DynamicDataSourceContextHolder.push(DatasourceConstant.SLAVE_NAME);
        fileInfoService.remove(q -> q
                .eq(FileInfo::getBusinessId, businessId)
                .eq(FileInfo::getBusinessType, businessType));

        DynamicDataSourceContextHolder.clear();
    }

    /**
     * 删除文件
     *
     * @param businessId 业务id
     */
    public static void removeFile(Long businessId) {
        DynamicDataSourceContextHolder.push(DatasourceConstant.SLAVE_NAME);
        fileInfoService.remove(q -> q.eq(FileInfo::getBusinessId, businessId));
        DynamicDataSourceContextHolder.clear();
    }

}
